/**
 *	This file is part of org.ow2.wildcat.
 *
 *	wildcat is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	wildcat is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	authors:
 *		- Pierre-Charles David (initial)
 *		- Nicolas Loriant
 */
package org.ow2.wildcat.hierarchy.resource;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ow2.wildcat.Context;
import org.ow2.wildcat.ContextFactory;
import org.ow2.wildcat.event.WHierarchyEvent;
import org.ow2.wildcat.hierarchy.Mountable;
import org.ow2.wildcat.hierarchy.Path;
import org.ow2.wildcat.hierarchy.attribute.Attribute;
import org.ow2.wildcat.util.Messages;

/**
 * Implement a local resource.
 */
public class LocalResource implements Resource {

	/**
	 * Logger.
	 */
	private static Log logger = LogFactory.getLog(LocalResource.class);

	private final Map<String, Resource> children;

	private final Map<String, Attribute> attributes;

	private Context context;

	private Path path;


	public LocalResource() {
		children = new HashMap<String, Resource>();
		attributes = new HashMap<String, Attribute>();
	}


	/**
	 * @param name
	 *            name of a resource
	 * @return <b>true</b> if a resource with given name is attached, else
	 *         <b>false</b>
	 */
	protected boolean hasResource(final String name) {
		return this.children.containsKey(name);
	}

	/**
	 * @param name
	 *            name of an attribute
	 * @return <b>true</b> if an attribute with given name is attached, else
	 *         <b>false</b>
	 */
	protected boolean hasAttribute(final String name) {
		return this.attributes.containsKey(name);
	}

	/**
	 * @param name
	 * @return the Resource with the given name
	 */
	protected Resource getResource(final String name) {
		return this.hasResource(name) ? this.children.get(name) : null;
	}

	/**
	 * @param name
	 * @return the Attribute with the given name
	 */
	protected Attribute getAttribute(final String name) {
		return this.hasAttribute(name) ? this.attributes.get(name) : null;
	}

	/**
	 * Attach an Attribute
	 *
	 * @param name
	 * @param attribute
	 */
	protected void attachAttribute(final String name, final Attribute attribute) {
		attribute.mount(this.context, this.path.appendAttribute(name));
		if (this.isMount()) {
			this.attributes.put(name, attribute);
			this.context.emitEvent(new WHierarchyEvent(this.path,
					WHierarchyEvent.Type.ATTRIBUTE_ADDED, name));
		}
	}

	/**
	 * @param attributeName
	 */
	private void detachAttribute(final String attributeName) {
		if(this.isMount()) {
			Attribute attribute = attributes.get(attributeName);
			attribute.unMount();
			attributes.remove(attributeName);
			context.emitEvent(new WHierarchyEvent(this.path,
					WHierarchyEvent.Type.ATTRIBUTE_REMOVED, attributeName));
		}
	}

	/**
	 * Attach a Resource
	 *
	 * @param name
	 * @param resource
	 */
	protected void attachResource(final String name, final Resource resource) {
		resource.mount(this.context, this.path.appendResource(name));
		if (this.isMount()) {
			this.children.put(name, resource);
			this.context.emitEvent(new WHierarchyEvent(this.path,
					WHierarchyEvent.Type.RESOURCE_ADDED, name));
		}
	}

	/**
	 * @param name
	 */
	private void detachResource(final String name) {
		if(isMount()) {
			Resource resource = children.get(name);
			resource.unMount();
			children.remove(resource);
			this.context.emitEvent(new WHierarchyEvent(this.path,
					WHierarchyEvent.Type.RESOURCE_REMOVED, name));
		}
	}

	/**
	 * Find or create an Resource
	 *
	 * @param name
	 * @param create true if a missing resource should be created
	 * @return the attached or created Resource, or <b>null</b> if there is no
	 *         Resource with given name but an Attribute already use that name
	 */
	protected Resource findOrCreate(final String name, final boolean create) {
		Resource r = getResource(name);
		if (r == null && !this.hasAttribute(name) && create) {
			r = ContextFactory.getDefaultFactory().createResource();
			this.attachResource(name, r);
		}
		return r;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#attachAttribute(org.ow2.wildcat.hierarchy.Path,
	 *      org.ow2.wildcat.hierarchy.attribute.Attribute)
	 */
	public void attachAttribute(final Path attribute, final Attribute attr) throws ResourceException {
		if (attribute.isOnlyAttribute()) {
			this.attachAttribute(attribute.getAttributeName(), attr);
		} else {
			Resource resource = this.findOrCreate(attribute.getFirstResource(), true);
			if (resource == null) {
				logger.info("Can't create resource \'"
						+ attribute.getFirstResource() + "\', ignoring");
				return;
			}
			resource.attachAttribute(attribute.stripTop(), attr);
		}
	}

	/**
	 * @param attribute
	 * @throws ResourceException
	 */
	public void detachAttribute(final Path attribute) throws ResourceException {
		if (attribute.isOnlyAttribute()) {
			this.detachAttribute(attribute.getAttributeName());
		} else {
			Resource resource = this.findOrCreate(attribute.getFirstResource(), false);
			if (resource == null) {
				logger.info("Can't create resource \'"
						+ attribute.getFirstResource() + "\', ignoring");
				return;
			}
			resource.detachAttribute(attribute.stripTop());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#attachResource(org.ow2.wildcat.hierarchy.Path,
	 *      org.ow2.wildcat.hierarchy.resource.Resource)
	 */
	public void attachResource(final Path resource, final Resource res) throws ResourceException {
		if (resource.isOnlyResource()) {
			this.attachResource(resource.getFirstResource(), res);
		} else {
			Resource r = this.findOrCreate(resource.getFirstResource(), true);
			if (r == null) {
				logger.info("Can't create resource \'"
						+ resource.getFirstResource() + "\', ignoring");
				return;
			}
			r.attachResource(resource.stripTop(), res);
		}
	}

	/**
	 * @param resource
	 * @throws ResourceException
	 */
	public void detachResource(final Path resource) throws ResourceException {
		if (resource.isOnlyResource()) {
			this.detachResource(resource.getFirstResource());
		} else {
			Resource r = this.findOrCreate(resource.getFirstResource(), false);
			if (r == null) {
				logger.info("Can't create resource \'"
						+ resource.getFirstResource() + "\', ignoring");
				return;
			}
			r.detachResource(resource.stripTop());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#attachAttribute(org.ow2.wildcat.hierarchy.Path,
	 *      org.ow2.wildcat.hierarchy.attribute.Attribute)
	 */
	public void createAttribute(final Path attribute, final Object value) throws ResourceException {
		if (attribute.isOnlyAttribute()) {
			Attribute a = ContextFactory.getDefaultFactory().createAttribute(value);
			this.attachAttribute(attribute.getAttributeName(), a);
		} else {
			Resource resource = this.findOrCreate(attribute.getFirstResource(), true);
			if (resource == null) {
				logger.info("Can't create resource \'"
						+ attribute.getFirstResource() + "\', ignoring");
				return;
			}
			resource.createAttribute(attribute.stripTop(), value);
		}
	}

	/**
	 * @param attribute
	 * @return
	 * @throws ResourceException
	 */
	public Attribute getAttribute(final Path attribute) throws ResourceException {
		if (attribute.isOnlyAttribute()) {
			return getAttribute(attribute.getAttributeName());
		} else {
			Resource resource = this.findOrCreate(attribute.getFirstResource(), false);
			if (resource == null) {
				logger.info("Can't create resource \'"
						+ attribute.getFirstResource() + "\', ignoring");
				return null;
			}
			return resource.getAttribute(attribute.stripTop());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#createResource(org.ow2.wildcat.hierarchy.Path)
	 */
	public void createResource(final Path resource) throws ResourceException {
		if (resource == null) {
			return;
		}
		Resource r = this.findOrCreate(resource.getFirstResource(), true);
		if (r == null) {
			logger.info("Can't create resource \'"
					+ resource.getFirstResource() + "\', ignoring");
			return;
		}
		if (resource.isOnlyResource()) {
			return;
		}
		r.createResource(resource.stripTop());
	}

	/**
	 * @param res
	 * @return
	 * @throws ResourceException
	 */
	public Resource getResource(final Path res) throws ResourceException {
		Resource r = this.findOrCreate(res.getFirstResource(), false);
		if(res.isOnlyResource()) {
			return r;
		} else {
			return r.getResource(res.stripTop());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#setValue(org.ow2.wildcat.hierarchy.Path,
	 *      java.lang.Object)
	 */
	public Object setValue(final Path attr, final Object value) throws ResourceException {
		if (attr.isOnlyAttribute()) {
			Attribute attribute = this.getAttribute(attr.getAttributeName());
			if (attribute == null) {
				logger.info("No attribute named '" + attr.getAttributeName()
						+ "', return 'null'");
				return null;
			}
			return attribute.setValue(value);
		} else {
			Resource resource = this.getResource(attr.getFirstResource());
			if (resource == null) {
				logger.info("No resource named '" + attr.getFirstResource()
						+ "', return 'null'");
				return null;
			}
			return resource.setValue(attr.stripTop(), value);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.Resource#getValue(org.ow2.wildcat.hierarchy.Path)
	 */
	public Object getValue(final Path attr) throws ResourceException {
		if (attr.isOnlyAttribute()) {
			Attribute attribute = this.getAttribute(attr.getAttributeName());
			if (attribute == null) {
				logger.info("No attribute named '" + attr.getAttributeName()
						+ "', return 'null'");
				return null;
			}
			return attribute.getValue();
		} else {
			Resource resource = this.getResource(attr.getFirstResource());
			if (resource == null) {
				logger.info("No resource named '" + attr.getFirstResource()
						+ "', return 'null'");
				return null;
			}
			return resource.getValue(attr.stripTop());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.resource.Resource#list(org.ow2.wildcat.hierarchy.Path)
	 */
	public Set<String> list(final Path resource) throws ResourceException {
		if (resource == null) {
			Set<String> r = new HashSet<String>();
			if (this.children != null) {
				r.addAll(this.children.keySet());
			}
			for (String s : this.attributes.keySet()) {
				r.add("#" + s);
			}
			return r;
		} else {
			Resource res = this.getResource(resource.getFirstResource());
			if (res == null) {
				logger.info("No resource named '" + resource.getFirstResource()
						+ "', return 'null'");
				return null;
			}
			if (resource.isOnlyResource()) {
				return res.list(null);
			} else {
				return res.list(resource.stripTop());
			}
		}
	}

	public boolean isMount() {
		return context != null && path != null;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.Mountable#mount(org.ow2.wildcat.Context,
	 *      org.ow2.wildcat.hierarchy.Path)
	 */
	public void mount(final Context context, final Path mountPoint) {
		logger.debug(Messages.message(Mountable.class.getName() + ".mount",
				mountPoint));

		if (this.isMount()) {
			logger.info(Messages.message(Mountable.class.getName()
					+ ".alreadymount", mountPoint));
		} else {
			this.context = context;
			this.path = mountPoint;
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.Mountable#unMount()
	 */
	public void unMount() {
		logger.debug(Messages.message(Mountable.class.getName() + ".unmount",
				(Object[]) null));

		if (this.children != null) {
			for (Map.Entry<String, Resource> entry : this.children.entrySet()) {
				entry.getValue().unMount();
				if (this.isMount()) {
					this.context.emitEvent(new WHierarchyEvent(this.path,
							WHierarchyEvent.Type.RESOURCE_REMOVED, entry
							.getKey()));
				}
			}
		}
		if (this.attributes != null) {
			for (Map.Entry<String, Attribute> entry : this.attributes
					.entrySet()) {
				entry.getValue().unMount();
				if (this.isMount()) {
					this.context.emitEvent(new WHierarchyEvent(this.path,
							WHierarchyEvent.Type.ATTRIBUTE_REMOVED, entry
							.getKey()));
				}
			}
		}
		this.context = null;
		this.path = null;
	}

}
