/**
 * WildCAT
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 EMN
 * Contact: wildcat@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DispatcherFactory.java 334 2008-09-05 17:14:19Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.wildcat.remote.dispatcher;

import java.io.InputStream;
import java.util.Properties;

import org.ow2.carol.util.configuration.CarolDefaultValues;
import org.ow2.carol.util.configuration.ConfigurationRepository;
import org.ow2.carol.util.configuration.ServerConfiguration;
import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.controller.server.AbsServerClusterViewManager;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.wildcat.ContextFactory;
import org.ow2.wildcat.remote.dispatcher.jms.JMSDispatcher;
import org.ow2.wildcat.remote.dispatcher.rmi.RMIDispatcher;
import org.ow2.wildcat.remote.provider.JMSProvider;
import org.ow2.wildcat.remote.provider.JMSProviderException;

/**
 * Factory to create a single instance of {@link IDispatcher}.
 * Dispatchers allow contexts to access to some remote contexts and listen them.
 * @see RMIDispatcher an implementation (this used by default) using RMI to pull and JMS to push.
 * @see JMSDispatcher an implementation using JMS to pull and push
 * @author Loris Bouzonnet
 */
public class DispatcherFactory {

	/**
	 * Logger.
	 */
	private static Log logger = LogFactory.getLog(DispatcherFactory.class);

	/**
	 * Singleton.
	 */
	private static IDispatcher dispatcher;

	/**
	 * Return a dispatcher of accesses to remote contexts.
	 * @param contextFactory a context factory to retrieve the configuration
	 * @return a dispatcher of accesses to remote contexts
	 * @throws DispatcherException if a dispatcher cannot be created
	 */
	public static IDispatcher getDispatcher(final ContextFactory contextFactory) throws DispatcherException {
		if (dispatcher == null) {

			boolean useCmi = !Boolean.parseBoolean(contextFactory.getProperty(ContextFactory.KEY_PURE_JMS));
			boolean cmiStarted = false;
			ServerConfiguration serverConfig;

			System.setProperty(CarolDefaultValues.SERVER_MODE, "true");
			try {
				ConfigurationRepository.init();
				serverConfig = ConfigurationRepository.getServerConfiguration();
			} catch (Exception e) {
				logger.error("Unable to init Carol!", e);
				throw new DispatcherException("Unable to init Carol!", e);
			}

			JMSProvider delegate = contextFactory.getJMSProviderDelegate();
			if(delegate != null) {
				try {
					delegate.init(contextFactory);
				} catch (JMSProviderException e) {
					logger.error("Unable to init the JMS provider delegate", e);
					throw new DispatcherException("Unable to init the JMS provider delegate", e);
				}
			}

			// Check if CMI is configured
			if(useCmi && !serverConfig.isStartCMI()) {
				InputStream stream =
					Thread.currentThread().getContextClassLoader().getResourceAsStream(
							contextFactory.getProperty(ContextFactory.KEY_CMI_CONFIG_FILENAME));
				Properties cmiProps = new Properties();
				try {
					cmiProps.load(stream);
					if(!serverConfig.isStartCMI()) {
						serverConfig.enableCMI(cmiProps);
					}
					cmiStarted = true;
					AbsServerClusterViewManager.getServerClusterViewManager().start(true);
				} catch(Exception e) {
					logger.info("Unable to configure CMI", e);
					useCmi = false;
				}
			}
			// Check if the CMI server is available
			if(useCmi && AbsServerClusterViewManager.getServerClusterViewManager().getState().equals(
					ClusterViewManager.State.AVAILABLE)) {
				dispatcher = new RMIDispatcher(contextFactory);
			} else {
				if(cmiStarted) {
					try {
						if(serverConfig.isStartCMI()) {
							serverConfig.disableCMI();
							AbsServerClusterViewManager.destroy();
						}
					} catch(Exception e) {
						logger.debug("Error while disabling CMI", e);
					}
				}
				// Else use pure JMS mode
				logger.info("Unable to use the RMI mode, using the pure JMS mode");
				dispatcher = new JMSDispatcher(contextFactory);
			}
		}
		return dispatcher;
	}

}
