/**
 * WildCAT
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 EMN
 * Contact: wildcat@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.wildcat.remote.provider.joram;

import java.net.ConnectException;
import java.util.HashMap;
import java.util.Map;

import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.Topic;
import javax.jms.TopicConnectionFactory;

import org.objectweb.joram.client.jms.Destination;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.local.QueueLocalConnectionFactory;
import org.objectweb.joram.client.jms.local.TopicLocalConnectionFactory;
import org.objectweb.joram.client.jms.tcp.QueueTcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TopicTcpConnectionFactory;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.wildcat.ContextFactory;
import org.ow2.wildcat.remote.provider.JMSProvider;
import org.ow2.wildcat.remote.provider.JMSProviderException;


/**
 * Provide a programmatic creation of a {@link Topic} with a JORAM collocated server (<a href="http://joram.ow2.org">http://joram.ow2.org</a>).
 * @author Loris Bouzonnet
 */
public final class JORAMProvider implements JMSProvider {

	/**
	 * Logger.
	 */
	private static Log logger = LogFactory.getLog(JORAMProvider.class);

	/**
	 * A collocated server (optional)
	 */
	private JORAMServer joramServer = null;

	/**
	 * Destinations created by this provider.
	 */
	private Map<String, Destination> createdDestinations = new HashMap<String, Destination>();

	private ContextFactory contextFactory;

	/**
	 * Connect to a JORAM server.
	 * Try to start a server if the connection failed.
	 * @param contextFactory a context factory to retrieve the configuration
	 * @throws JMSProviderException if a connection to a JORAM server cannot be performed
	 */
	public void init(final ContextFactory contextFactory) throws JMSProviderException {

		this.contextFactory = contextFactory;

		String hostname = contextFactory.getProperty(ContextFactory.KEY_JORAM_SERVER_HOSTNAME);
		int port = Integer.parseInt(contextFactory.getProperty(ContextFactory.KEY_JORAM_SERVER_PORT));

		// Check if the JORAM server is available
		if(!isJORAMServerAvailable()) {
			if(contextFactory.isEmbedded()) {
				// Start a collocated JORAM server
				short id = Short.parseShort(contextFactory.getProperty(ContextFactory.KEY_JORAM_SERVER_ID));
				joramServer = new JORAMServer(id, hostname, port,
						contextFactory.getProperty(ContextFactory.KEY_TOPIC_CONNECTION_FACTORY),
						contextFactory.getProperty(ContextFactory.KEY_QUEUE_CONNECTION_FACTORY));
				try {
					joramServer.start();
				} catch (Exception e) {
					logger.error("Cannot start the JORAM server", e);
					throw new JMSProviderException("Cannot start the JORAM server", e);
				}
			} else  {
				logger.error("Cannot connect to the admin module");
				throw new JMSProviderException(
				"Cannot connect to the admin module");
			}
		}
	}

	/**
	 * Return true if a connection to a JORAM server can be performed.
	 * @return true if a connection to a JORAM server can be performed
	 */
	private boolean isJORAMServerAvailable() {
		if(contextFactory.isEmbedded()) {
			try {
				AdminModule.collocatedConnect("root", "root");
			} catch (Exception e) {
				logger.info("Cannot connect to the collocated JORAM server");
				logger.debug("cause: ", e);
				return false;
			}
		} else {
			String hostname = contextFactory.getProperty(ContextFactory.KEY_JORAM_SERVER_HOSTNAME);
			int port = Integer.parseInt(contextFactory.getProperty(ContextFactory.KEY_JORAM_SERVER_PORT));
			try {
				AdminModule.connect(hostname, port, "root", "root", 10);
			} catch (Exception e) {
				logger.info("Cannot connect to the JORAM server at host {0} with port {1}", hostname, port);
				logger.debug("cause: ", e);
				return false;
			}
		}
		return true;
	}

	/**
	 * @return a topic connection factory
	 * @throws JMSProviderException if a topic connection factory cannot be created
	 */
	public TopicConnectionFactory getTopicConnectionFactory() throws JMSProviderException {
		if(contextFactory.isEmbedded()) {
			return TopicLocalConnectionFactory.create();
		} else {
			try {
				return TopicTcpConnectionFactory.create();
			} catch (ConnectException e) {
				logger.error("Unable to create a topic connection factory", e);
				throw new JMSProviderException("Unable to create a topic connection factory", e);
			}
		}
	}

	/**
	 * Create a {@link Topic} by directly using the JORAM {@link AdminModule}.
	 * @param topicName a topic name
	 * @return a topic with the given name
	 * @throws JMSProviderException if the topic cannot be created
	 */
	public Topic getTopic(final String topicName) throws JMSProviderException {

		org.objectweb.joram.client.jms.Topic topic;
		try {
			topic = org.objectweb.joram.client.jms.Topic.create(topicName);
			topic.setFreeReading();
			topic.setFreeWriting();
			createdDestinations.put(topicName, topic);
		} catch (Exception e) {
			logger.error("Cannot create the topic with name {0}", topicName, e);
			throw new JMSProviderException("Cannot create the topic with name " + topicName, e);
		}
		return topic;
	}

	/**
	 * Delete the created {@link Topic} and stop the collocated server.
	 */
	public void destroy() {
		for(Destination dest : createdDestinations.values()) {
			try {
				dest.delete();
			} catch (Exception e) {
				logger.warn("Unable to delete the destination with name " + dest.getAdminName(), e);
			}
		}
		createdDestinations.clear();
		if(joramServer != null) {
			try {
				joramServer.stop();
			} catch (Exception e) {
				logger.warn("Unable to stop the JORAM collocated server", e);
			}
		}
	}

	/**
	 * Create a {@link Queue} by directly using the JORAM {@link AdminModule}.
	 * @param queueName a queue name
	 * @return a queue with the given name
	 * @throws JMSProviderException if the queue cannot be created
	 */
	public Queue getQueue(final String queueName) throws JMSProviderException {
		org.objectweb.joram.client.jms.Queue queue;
		try {
			queue = org.objectweb.joram.client.jms.Queue.create(queueName);
			queue.setFreeReading();
			queue.setFreeWriting();
			createdDestinations.put(queueName, queue);
		} catch (Exception e) {
			logger.error("Cannot create the queue with name {0}", queueName, e);
			throw new JMSProviderException("Cannot create the queue with name " + queueName, e);
		}
		return queue;
	}

	/**
	 * @return a queue connection factory
	 * @throws JMSProviderException if a topic connection factory cannot be created
	 */
	public QueueConnectionFactory getQueueConnectionFactory()
	throws JMSProviderException {
		if(contextFactory.isEmbedded()) {
			return QueueLocalConnectionFactory.create();
		} else {
			try {
				return QueueTcpConnectionFactory.create();
			} catch (ConnectException e) {
				logger.error("Unable to create a queue connection factory", e);
				throw new JMSProviderException("Unable to create a queue connection factory", e);
			}
		}
	}

	/**
	 * Delete the queue with the given name.
	 * @param queueName a queue name
	 * @throws JMSProviderException if the queue cannot be removed
	 */
	public void deleteQueue(final String queueName) throws JMSProviderException {
		if(createdDestinations.containsKey(queueName)) {
			try {
				createdDestinations.remove(queueName).delete();
			} catch (Exception e) {
				logger.error("Unable to remove the queue with name " + queueName, e);
				throw new JMSProviderException("Unable to remove the queue with name " + queueName, e);
			}
		}
	}

	/**
	 * Delete the topic with given name.
	 * @param topicName a topic name
	 * @throws JMSProviderException if the topic cannot be removed
	 */
	public void deleteTopic(final String topicName) throws JMSProviderException {
		if(createdDestinations.containsKey(topicName)) {
			try {
				createdDestinations.remove(topicName).delete();
			} catch (Exception e) {
				logger.error("Unable to remove the topic with name " + topicName, e);
				throw new JMSProviderException("Unable to remove the topic with name " + topicName, e);
			}
		}
	}

}
