/**
 *	This file is part of org.ow2.wildcat.
 *
 *	wildcat is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	wildcat is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	authors:
 *		- Pierre-Charles David (initial)
 *		- Nicolas Loriant
 */
package org.ow2.wildcat.sensors;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.text.Format;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Map;

import org.ow2.wildcat.hierarchy.attribute.POJOAttribute;

/**
 * Abstract class for /proc sampled sensors
 */
public abstract class TextFileSensor extends POJOAttribute implements Sensor {

	/**
	 * Id for serializable class.
	 */
	private static final long serialVersionUID = -6488447539127928360L;

	protected String filename;

	/**
	 * Constructor
	 *
	 * @param filename
	 *            full path to sampled file
	 */
	protected TextFileSensor(final String filename) {
		super(null);
		this.filename = filename;
	}

	/**
	 * Transform parses lines into a Map
	 *
	 * @param lines
	 *            full text lines
	 * @return parsed lines
	 */
	protected abstract Map<String, Object> getValues(String[] lines);

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.attribute.POJOAttribute#getValue()
	 */
	@Override
	public Object getValue() {
		return this.getValues(this.getLines());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.wildcat.hierarchy.attribute.POJOAttribute#setValue(java.lang.Object)
	 */
	@Override
	public Object setValue(final Object value) {
		return this.getValue();
	}

	protected String[] getLines() {
		ArrayList<String> lines = new ArrayList<String>();
		try {
			LineNumberReader reader = null;
			try {
				reader = new LineNumberReader(new FileReader(filename));
				String line = null;
				while ((line = reader.readLine()) != null) {
					lines.add(line);
				}
			} catch (FileNotFoundException e) {
				throw new IOException("Unable to open " + filename + " file: "
						+ e.getMessage());
			} finally {
				if (reader != null) {
					reader.close();
				}
			}
		} catch (IOException e) {
			lines.clear();
		}
		return lines.toArray(new String[lines.size()]);
	}

	protected static final class LineFormat {
		private Format format;
		private String property;

		public LineFormat(final String property, final String format) {
			this(property, new MessageFormat(format));
		}

		public LineFormat(final String property, final Format format) {
			this.property = property;
			this.format = format;
		}

		public Format getFormat() {
			return format;
		}

		public String getProperty() {
			return property;
		}
	}
}
